@@ -96,7 +96,7 @@
     }else{
       params = parseParams();
     }
-    return $('link[rel=kss-base-url]').attr('href') + $o.url + '?' + params;
+    return $('base').attr('href') + $o.url + '?' + params;
     //return $o.url+'?view_name='+tabbedview.prop('view_name');
     //return $o.url+'?show='+methods.param('show')+'&path='+methods.param('path');
   }
@@ -151,7 +151,17 @@
   };
 
   $.fn.ftwtable.reloadTable = function(table, query, options){
-    $.fn.ftwtable.createTable($this, buildQuery(), $o);
+    if(typeof(tabbedview) != "undefined") {
+      tabbedview.show_spinner();
+    }
+
+    $this.load(query, null, function() {
+      tabbedview.view_container.trigger('gridRendered');
+      if(typeof(tabbedview) != "undefined") {
+        tabbedview.hide_spinner();
+      }
+      $o.onLoad();
+    });
   };
 
   $.fn.ftwtable.select = function(start, end){
