@@ -8,6 +8,43 @@ import cgi
 import os.path
 
 
+def link(icon=True, tooltip=False, classes=None, attrs=None):
+    """Generates a helper.
+
+    Attributes:
+    icon -- Shows a content type icon.
+    tooltip -- Adds a "title" attribute and a "rollover" class.
+    classes -- A list of additional classes.
+    attrs -- A dict of additional attributes.
+    """
+    if classes is None:
+        classes = []
+    if attrs is None:
+        attrs = {}
+
+    assert isinstance(classes, list)
+    assert isinstance(attrs, dict)
+
+    def _helper(item, value):
+        attributes = attrs.copy()
+        attributes['class'] = classes[:]
+
+        if tooltip:
+            description = item.Description
+            if callable(description):
+                description = description()
+
+            if description:
+                if isinstance(description, str):
+                    description = description.decode('utf-8')
+
+                attributes['class'].append('rollover')
+                attributes['title'] = cgi.escape(description, quote=True)
+
+        return linked(item, value, show_icon=icon, attrs=attributes)
+    return _helper
+
+
 def draggable(item, value):
     return '<span id="draggable-%s" class="draggable">::</span>' % item.id
 
@@ -54,6 +91,12 @@ def readable_author(item, author):
         name = user.getProperty('fullname', author) or author
         if not len(name):
             name = author
+
+    if isinstance(name, unicode):
+        name = name.encode('utf-8')
+    if isinstance(author, unicode):
+        author = author.encode('utf-8')
+
     return '<a href="%s/author/%s">%s</a>' % (portal_url(), author, name)
 
 
@@ -61,7 +104,7 @@ def readable_date_time_text(item, date):
     today = datetime.today().strftime('%Y%m%d')
     yesterday = (datetime.today() - timedelta(1)).strftime('%Y%m%d')
     strftimestring = '%d.%m.%Y %H:%M'
-    if date == None:
+    if date is None:
         return None
     if not getattr(date, 'strftime', None):
         return None
@@ -74,7 +117,7 @@ def readable_date_time_text(item, date):
 
 def readable_date_time(item, date):
     strftimestring = '%d.%m.%Y %H:%M'
-    if date == None:
+    if date is None:
         return None
     try:
         return date.strftime(strftimestring)
@@ -86,7 +129,7 @@ def readable_date_text(item, date):
     today = datetime.today().strftime('%Y%m%d')
     yesterday = (datetime.today() - timedelta(1)).strftime('%Y%m%d')
     strftimestring = '%d.%m.%Y'
-    if date == None:
+    if date is None:
         return None
     if not getattr(date, 'strftime', None):
         return None
@@ -107,7 +150,19 @@ def readable_date(item, date):
         return None
 
 
-def linked(item, value, show_icon=True):
+def linked(item, value, show_icon=True, attrs=None):
+    if attrs is None:
+        attrs = {}
+
+    if 'class' not in attrs:
+        attrs['class'] = []
+    elif isinstance(attrs['class'], str):
+        attrs['class'] = attrs['class'].decode('utf-8').strip().split(' ')
+    elif isinstance(attrs, unicode):
+        attrs['class'] = attrs['class'].strip().split(' ')
+    else:
+        attrs['class'] = list(attrs['class'])
+
     url_method = lambda: '#'
     #item = hasattr(item, 'aq_explicit') and item.aq_explicit or item
     if hasattr(item, 'getURL'):
@@ -115,19 +170,24 @@ def linked(item, value, show_icon=True):
     elif hasattr(item, 'absolute_url'):
         url_method = item.absolute_url
 
-    type_class = ''
     if show_icon:
         site = getSite()
         plone_utils = getToolByName(site, 'plone_utils')
         portal_url = getToolByName(site, 'portal_url')
-        img = u'<img src="%s/%s"/>' % (
-            portal_url(), item.getIcon)
-        if not item.getIcon:
-            type_class = ' class="contenttype-%s"' % \
-                plone_utils.normalizeString(item.portal_type)
+
+        icon = item.getIcon
+        if callable(icon):
+            icon = icon()
+
+        img = u'<img src="%s/%s"/>' % (portal_url(), icon)
+        if not icon:
+            attrs['class'].append(
+                'contenttype-%s' %
+                plone_utils.normalizeString(item.portal_type))
             img = u''
     else:
         img = u''
+
     if not isinstance(value, unicode):
         value = value.decode('utf8')
     value = cgi.escape(value, quote=True)
@@ -142,8 +202,17 @@ def linked(item, value, show_icon=True):
         if item.portal_type in types_using_view:
             href = os.path.join(href, 'view')
 
-    link = u'<a href="%s"%s>%s%s</a>' % (
-        href, type_class, img, value)
+    attrs['href'] = href
+
+    if attrs['class']:
+        attrs['class'] = ' '.join(sorted(set(attrs['class'])))
+    else:
+        del attrs['class']
+    attrs_str = ' '.join(
+        ['%s="%s"' % (attrkey, attrvalue) for attrkey, attrvalue in
+         sorted(attrs.items())])
+
+    link = u'<a %s>%s%s</a>' % (attrs_str, img, value)
     wrapper = u'<span class="linkWrapper">%s</span>' % link
     return wrapper
 
